@@ -328,7 +328,7 @@ int main(int argc, char **argv)
 		exit(1);
 
 	/*
-	 * Override debug level
+	 * Init debug level with -d option value
 	 */
 	pool_config->debug_level = debug_level;
 
@@ -348,12 +348,6 @@ int main(int argc, char **argv)
 		pool_config->logsyslog = 1;
 	}
 
-	/*
-	 * Override debug level
-	 */
-	if (pool_config->debug_level == 0)
-		pool_config->debug_level = debug_level;
-
 	if (pool_config->enable_pool_hba)
 		load_hba(hba_file);
 
@@ -2858,7 +2852,14 @@ static int find_primary_node(void)
 		if (!s)
 		{
 			pool_error("find_primary_node: make_persistent_connection failed");
-			return -1;
+
+			/*
+			 * It is possible that a node is down even if
+			 * VALID_BACKEND tells it's valid.  This could happen
+			 * before health checking detects the failure.
+			 * Thus we should continue to look for primary node.
+			 */
+			continue;
 		}
 		con = s->con;
 		status = do_query(con, "SELECT pg_is_in_recovery()",
