@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2013	PgPool Global Development Group
+ * Copyright (c) 2003-2014	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -657,7 +657,7 @@ bool is_unlogged_table(char *table_name)
 		{
 			relcache = pool_create_relcache(pool_config->relcache_size, query,
 											int_register_func, int_unregister_func,
-											true);
+											false);
 			if (relcache == NULL)
 			{
 				pool_error("is_unlogged_table: pool_create_relcache error");
@@ -683,11 +683,11 @@ bool is_unlogged_table(char *table_name)
 bool is_view(char *table_name)
 {
 /*
- * Query to know if the target table is a view.
+ * Query to know if the target table is a view (including a materialized view).
  */
-#define ISVIEWQUERY "SELECT count(*) FROM pg_catalog.pg_class AS c WHERE c.relname = '%s' AND c.relkind = 'v'"
+#define ISVIEWQUERY "SELECT count(*) FROM pg_catalog.pg_class AS c WHERE c.relname = '%s' AND (c.relkind = 'v' OR c.relkind = 'm')"
 
-#define ISVIEWQUERY2 "SELECT count(*) FROM pg_catalog.pg_class AS c WHERE c.oid = pgpool_regclass('%s') AND c.relkind = 'v'"
+#define ISVIEWQUERY2 "SELECT count(*) FROM pg_catalog.pg_class AS c WHERE c.oid = pgpool_regclass('%s') AND (c.relkind = 'v' OR c.relkind = 'm')"
 
 	static POOL_RELCACHE *relcache;
 	POOL_CONNECTION_POOL *backend;
